@@ -24,40 +24,40 @@ O_REGEX = r'`([^`\$]+)`'
 class Style:
     end_mark = '\033[0m'
 
-    @classmethod
-    def purple(cls, text):
-        return '\033[95m' + text + cls.end_mark
-
-    @classmethod
-    def dark_cyan(cls, text):
-        return '\033[36m' + text + cls.end_mark
-
-    @classmethod
-    def cyan(cls, text):
-        return '\033[96m' + text + cls.end_mark
-
-    @classmethod
-    def blue(cls, text):
-        return '\033[94m' + text + cls.end_mark
-
-    @classmethod
-    def green(cls, text):
-        return '\033[92m' + text + cls.end_mark
-
-    @classmethod
-    def yellow(cls, text):
-        return '\033[93m' + text + cls.end_mark
-
-    @classmethod
-    def red(cls, text):
-        return '\033[91m' + text + cls.end_mark
-
     @classmethod
     def bold(cls, text):
         return '\033[1m' + text + cls.end_mark
 
     @classmethod
-    def underline(cls, text):
+    def purple(cls, text):  # NO COVER: not in use
+        return '\033[95m' + text + cls.end_mark
+
+    @classmethod
+    def dark_cyan(cls, text):  # NO COVER: not in use
+        return '\033[36m' + text + cls.end_mark
+
+    @classmethod
+    def cyan(cls, text):  # NO COVER: not in use
+        return '\033[96m' + text + cls.end_mark
+
+    @classmethod
+    def blue(cls, text):  # NO COVER: not in use
+        return '\033[94m' + text + cls.end_mark
+
+    @classmethod
+    def green(cls, text):  # NO COVER: not in use
+        return '\033[92m' + text + cls.end_mark
+
+    @classmethod
+    def yellow(cls, text):  # NO COVER: not in use
+        return '\033[93m' + text + cls.end_mark
+
+    @classmethod
+    def red(cls, text):  # NO COVER: not in use
+        return '\033[91m' + text + cls.end_mark
+
+    @classmethod
+    def underline(cls, text):  # NO COVER: not in use
         return '\033[1m' + text + cls.end_mark
 
 
@@ -111,9 +111,10 @@ def decorate(target, decorators):
     return '\n'.join([f'@{decorator}' for decorator in decorators] + [target])
 
 
-def make_def_content(*doc_lines, body=''):
-    return indent('\n'.join(([make_doc(*doc_lines)] if doc_lines else []) +
-                            ([body] if body else []) or ['pass']))
+def make_def_content(*doc_lines, body='', upper_blank_line=True):
+    sep = '\n\n' if upper_blank_line else '\n'
+    return indent(sep.join(([make_doc(*doc_lines)] if doc_lines else []) +
+                           ([body.strip()] if body.strip() else []) or ['pass']))
 
 
 def make_class_head(name, bases=(), decorators=()):
@@ -127,9 +128,11 @@ def make_class(name, *doc_lines, bases=(), decorators=(), body=''):
                       make_def_content(*doc_lines, body=body)])
 
 
-def make_method(name, *doc_lines, args_text='', decorators=(), body=''):
-    return '\n'.join([decorate(f'def {name}(self{args_text}):', decorators),
-                      make_def_content(*doc_lines, body=body)])
+def make_method(name, *doc_lines, args_text='self', decorators=(), body='',
+                upper_blank_line=True):
+    return '\n'.join([decorate(f'def {name}({args_text}):', decorators),
+                      make_def_content(*doc_lines, body=body,
+                                       upper_blank_line=upper_blank_line)])
 
 
 def rstrip(text):
@@ -137,7 +140,7 @@ def rstrip(text):
 
 
 def assert_test_files_match(origin_dir, target_dir):
-    py_file_names = ['__init__.py', 'aliases.py', 'base.py', 'test_stories.py']
+    py_file_names = ['__init__.py', 'base.py', 'test_stories.py']
     missing_in_origin = set(py_file_names) - set(os.listdir(origin_dir))
     missing_in_target = set(py_file_names) - set(os.listdir(target_dir))
 
